function Vincenty_Distance(lat1,lon1,lat2,lon2,us,meters_only) {
	// http://www.movable-type.co.uk/scripts/LatLongVincenty.html
	if (Math.abs(parseFloat(lat1)) > 90 || Math.abs(parseFloat(lon1)) > 180 || Math.abs(parseFloat(lat2)) > 90 || Math.abs(parseFloat(lon2)) > 180) { return 'n/a'; }
	if (lat1 == lat2 && lon1 == lon2) { return '0'; }
	
	lat1 = deg2rad(lat1); lon1 = deg2rad(lon1);
	lat2 = deg2rad(lat2); lon2 = deg2rad(lon2);

	var a = 6378137, b = 6356752.3142, f = 1/298.257223563;
	var L = lon2 - lon1;
	var U1 = Math.atan((1-f) * Math.tan(lat1));
	var U2 = Math.atan((1-f) * Math.tan(lat2));
	var sinU1 = Math.sin(U1), cosU1 = Math.cos(U1);
	var sinU2 = Math.sin(U2), cosU2 = Math.cos(U2);
	var lambda = L, lambdaP = 2*Math.PI;
	var iterLimit = 50;
	while (Math.abs(lambda-lambdaP) > 1e-12 && --iterLimit > 0) {
		var sinLambda = Math.sin(lambda), cosLambda = Math.cos(lambda);
		var sinSigma = Math.sqrt((cosU2*sinLambda) * (cosU2*sinLambda) + 
		  (cosU1*sinU2-sinU1*cosU2*cosLambda) * (cosU1*sinU2-sinU1*cosU2*cosLambda));
		var cosSigma = sinU1*sinU2 + cosU1*cosU2*cosLambda;
		var sigma = Math.atan2(sinSigma, cosSigma);
		var alpha = Math.asin(cosU1 * cosU2 * sinLambda / sinSigma);
		var cosSqAlpha = Math.cos(alpha) * Math.cos(alpha);
		var cos2SigmaM = (!cosSqAlpha) ? 0 : cosSigma - 2*sinU1*sinU2/cosSqAlpha;
		var C = f/16*cosSqAlpha*(4+f*(4-3*cosSqAlpha));
		lambdaP = lambda;
		lambda = L + (1-C) * f * Math.sin(alpha) * (sigma + C*sinSigma*(cos2SigmaM+C*cosSigma*(-1+2*cos2SigmaM*cos2SigmaM)));
	}
	if (iterLimit==0) { return (NaN); }  // formula failed to converge
	var uSq = cosSqAlpha*(a*a-b*b)/(b*b);
	var A = 1 + uSq/16384*(4096+uSq*(-768+uSq*(320-175*uSq)));
	var B = uSq/1024 * (256+uSq*(-128+uSq*(74-47*uSq)));
	var deltaSigma = B*sinSigma*(cos2SigmaM+B/4*(cosSigma*(-1+2*cos2SigmaM*cos2SigmaM) - B/6*cos2SigmaM*(-3+4*sinSigma*sinSigma)*(-3+4*cos2SigmaM*cos2SigmaM)));
	var s = b*A*(sigma-deltaSigma);
	
	if (meters_only) {
		return s;
	} else if (us) {
		var dist = s / 1609.344;
		if (dist < 0.2) {
			return (Math.round(5280 * 1 * dist) / 1) + ' ft';
		} else {
			return (Math.round(1000 * dist) / 1000) + ' mi';
		}
	} else {
		var dist = s / 1000;
		if (dist < 1) {
			return (Math.round(1000 * 1 * dist) / 1) + ' m';
		} else {
			return (Math.round(1000 * dist) / 1000) + ' km';
		}
	}
}


function parseCoordinate(coordinate,type,format,spaced)
{
	coordinate = coordinate.toString();
	coordinate = coordinate.replace(/(^\s+|\s+$)/g,''); // remove white space
	var neg = 0; if (coordinate.match(/(^-|[WS])/i)) { neg = 1; }
	if (coordinate.match(/[EW]/i) && !type) { type = 'lon'; }
	if (coordinate.match(/[NS]/i) && !type) { type = 'lat'; }
	coordinate = coordinate.replace(/[NESW\-]/gi,' ');
	if (!coordinate.match(/[0-9]/i)) {
		return '';
	}
	parts = coordinate.match(/([0-9\.\-]+)[^0-9\.]*([0-9\.]+)?[^0-9\.]*([0-9\.]+)?/);
	if (!parts || parts[1] == null) {
		return '';
	} else {
		n = parseFloat(parts[1]);
		if (parts[2]) { n = n + parseFloat(parts[2])/60; }
		if (parts[3]) { n = n + parseFloat(parts[3])/3600; }
		if (neg && n >= 0) { n = 0 - n; }
		if (format == 'dmm') {
			if (spaced) {
				n = Degrees_to_DMM(n,type,' ');
			} else {
				n = Degrees_to_DMM(n,type);
			}
		} else if (format == 'dms') {
			if (spaced) {
				n = Degrees_to_DMS(n,type,' ');
			} else {
				n = Degrees_to_DMS(n,type,'');
			}
		} else {
			n = Math.round(10000000 * n) / 10000000;
			if (n == Math.floor(n)) { n = n + '.0'; }
		}
		return comma2point(n);
	}
}

function parseDistance(distance_text)
{ // returns meters
	var meters = parseFloat(distance_text.replace(/^.*?([0-9]+\.?[0-9]*).*$/,"$1"));
	if (distance_text.match(/mi/i)) { meters *= 1609.344; }
	else if (distance_text.match(/(\b|\d)(m\b|meter)/i)) { meters *= 1; }
	else if (distance_text.match(/(\b|\d)(naut|n\.?m|kn)/i)) { meters *= 1852; }
	else if (distance_text.match(/((\b|\d)fe*t|')/i)) { meters *= 0.3048; }
	else { meters *= 1000; } // default is kilometers
	return (meters);
}

function parseBearing(bearing_text)
{ // returns degrees
	var degrees;
	if (bearing_text.toUpperCase().match(/[NS].*[0-9].*[EW]/i)) {
		parts = bearing_text.toUpperCase().match(/([NS])(.*[0-9].*)([EW])/);
		degrees = parts[2];
		if (parts[1] == 'N' && parts[3] == 'E') { degrees = 0 + parseCoordinate(degrees); }
		else if (parts[1] == 'N' && parts[3] == 'W') { degrees = 360 - parseCoordinate(degrees); }
		else if (parts[1] == 'S' && parts[3] == 'E') { degrees = 180 - parseCoordinate(degrees); }
		else if (parts[1] == 'S' && parts[3] == 'W') { degrees = 180 + parseCoordinate(degrees); }
	} else {
		degrees = parseCoordinate(bearing_text.replace(/[NSEW]/gi,' '));
	}
	return degrees;
}

function comma2point (number)
{
   number = number+''; // force number into a string context
   return (number.replace(/,/g,'.'));
}

function deg2rad (deg)
{
  return (parseFloat(comma2point(deg)) * 3.14159265358979/180);
}

function rad2deg (radians)
{
  return (Math.round(10000000 * parseFloat(radians) * 180/3.14159265358979) / 10000000);
}

function Point_At_Distance_And_Bearing(start_lat,start_lon,distance_text,bearing)
{ // input is in degrees, km, degrees
// http://www.fcc.gov/mb/audio/bickel/sprong.html
   var earth_radius = 6378137; // equatorial radius
// var earth_radius = 6356752; // polar radius
// var earth_radius = 6371000; // typical radius

   var distance = parseDistance(distance_text);
   bearing = parseBearing(bearing);
   var start_lat_rad = deg2rad(parseCoordinate(start_lat));
   var start_lon_rad = deg2rad(parseCoordinate(start_lon));
   var ending_point = []; // output
   var isig = 0; // azimuth < 180 degrees
   var a = (360-bearing) % 360;
   if (a > 180) { a = 360 - a; isig = 1; } // azimuth > 180 degrees
   var a = deg2rad(a); var bb = (Math.PI/2)-start_lat_rad; var cc = distance/earth_radius;
   var sin_bb = Math.sin(bb); var cos_bb = Math.cos(bb); var cos_cc = Math.cos(cc);
   var cos_aa = cos_bb*cos_cc+(sin_bb*Math.sin(cc)*Math.cos(a));
   if (cos_aa <= -1) { cos_aa = -1; } if (cos_aa >= 1) { cos_aa = 1; }
   var aa = (cos_aa.toFixed(15) == 1) ? 0 : Math.acos(cos_aa);
   var cos_c = (cos_cc-(cos_aa*cos_bb))/(Math.sin(aa)*sin_bb);
   if (cos_c <= -1) { cos_c = -1; } if (cos_c >= 1) { cos_c = 1; }
   var c = (cos_c.toFixed(15) == 1) ? 0 : Math.acos(cos_c);
   var end_lat_rad = (Math.PI/2)-aa;
   var end_lon_rad = start_lon_rad-c;
   if (isig == 1) { end_lon_rad = start_lon_rad + c; }
   if (end_lon_rad > Math.PI) { end_lon_rad = end_lon_rad - (2*Math.PI); }
   if (end_lon_rad < (0-Math.PI)) { end_lon_rad = end_lon_rad + (2*Math.PI); }
   ending_point[0] = rad2deg(end_lat_rad); ending_point[1] = rad2deg(end_lon_rad);
   // Use proportional error to adjust things due to oblate Earth; I'm still not entirely sure how/why this works:
   for (i=0; i<5; i++)
   {
      var vincenty = Vincenty_Distance(start_lat,start_lon,ending_point[0],ending_point[1],false,true);
      if (Math.abs(start_lon-ending_point[1]) > 90)
      {
	 // something went haywire	
      }
      else
      { 
         var error = (vincenty != 0) ? distance/vincenty : 1;
	 var dlat = ending_point[0]-parseFloat(start_lat); var dlon = ending_point[1]-parseFloat(start_lon);
			ending_point[0] = parseFloat(start_lat)+(dlat*error); ending_point[1] = parseFloat(start_lon)+(dlon*error);
      }
   }
   return (ending_point);
}

function distanceFromLatLon(fromLat,fromLon,toLat,toLon)
{
var R = 6371; // km
var lat1 = fromLat.toRad();
var lon1 = fromLon.toRad();
var lat2 = toLat.toRad();
var lon2 = toLon.toRad();
var dLat = lat2-lat1;
var dLon = lon2-lon1; 
var a = Math.sin(dLat/2) * Math.sin(dLat/2) +
        Math.cos(lat1) * Math.cos(lat2) * 
        Math.sin(dLon/2) * Math.sin(dLon/2); 
var c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a)); 
var d= R * c;
return d.toFixed(3);
}

function bearingFromLatLon(fromLat,fromLon,toLat,toLon)
{
var lat1 = fromLat.toRad();
var lat2 = toLat.toRad();
var dLon = (toLon-fromLon).toRad(); 
var y = Math.sin(dLon) * Math.cos(lat2);
var x = Math.cos(lat1)*Math.sin(lat2) -
        Math.sin(lat1)*Math.cos(lat2)*Math.cos(dLon);
var brng = Math.atan2(y, x);
return (brng.toDeg()+360) % 360;
}

Number.prototype.toRad = function() {  // convert degrees to radians
  return this * Math.PI / 180;
}

Number.prototype.toDeg = function() {  // convert radians to degrees (signed)
  return this * 180 / Math.PI;
}

Number.prototype.toBrng = function() {  // convert radians to degrees (as bearing: 0...360)
  return (this.toDeg()+360) % 360;
}
